#!/usr/bin/perl -w

# HTML::HashTable
#
# Kirrily 'Skud' Robert <skud@netizen.com.au>
#
# Changelog
#
# 1999-09-11	Initial release
# 1999-07-??	Original script written at 4am for no good reason
#
# Version 0.5
#

$VERSION = $VERSION = 0.5;

require 5;

package HTML::HashTable;

require Exporter;
@ISA = qw( Exporter );

=head1 NAME

C<HTML::HashTable> - Create an HTML table from a Perl hash

=head1 SYNOPSIS

    use HTML::HashTable
    $output .= tablify({BORDER => 0, 
	DATA => $myhashref, 
	SORTBY => 'key', 
	ORDER => 'desc'});

HTML::HashTable is available at a CPAN site near you.

=head1 DESCRIPTION

This module takes an arbitrary Perl hash and presents it as an HTML
table.  The hash can contain anything you want -- scalar data, 
arrayrefs, hashrefs, whatever.  Yes, this means you can use a tied
hash if you wish.

The HTML produced is nicely formatted and indented, suitable for
human editing and manipulation.

Some options are provided with the tablify() function to allow you
to specify whether you wish to have a border or not, and whether you
wish your table to be sorted by key or value (but note that sorting
by value gives almost meaningless results if your values are 
references)

The options given to the tablify() function are:

=item C<BORDER>

True or false depending on whether you want your table to have a
border

=item C<DATA>

Reference to your hash

=item C<SORTBY>

Either 'key' or 'value' depending on how you want your data sorted.
Note that sorting by value is more or less meaningless if your
values are referenes.

=item C<ORDER>

Either 'asc' or 'desc' depending on whether you want your sorting to
be in ascending or descending order.

=head1 BUGS

Tell me about them, please.

=head1 FUTURE DIRECTIONS

I intend that in the (reasonably near) future this module will support
options to "prettify" the HTML generated.  However, I will avoid any
constructs which are not valid HTML.  So expect to see support for 
class attributes before you see Netscape/Microsoft HTML "enhancements".

=head1 AUTHOR

Kirrily 'Skud' Robert

Netizen Pty Ltd

skud@netizen.com.au



=cut

@EXPORT = qw(tablify);

#
# The main subroutine
#

sub tablify {
	$output = '';
	$depth = 0;
	my $tsref = shift;
	make_table($tsref);
	return $output;
}

#
# This subroutine does most of the work by recursing through the
# hash supplied.  We look to see whether the value of any hash
# item is a scalar, an arrayref or a hashref, and act accordingly.
# Recursion's so rare in Perl... this is *fun*
#

sub recurse_through {
	my $tsref = shift;
	my $thingy = shift;
	if (ref($thingy) eq 'ARRAY') {
		foreach (@$thingy) {
			recurse_through($tsref, $_);
		}
	} elsif (ref($thingy) eq 'HASH') {
		my $newref = {%$tsref};
		$newref->{DATA} = $thingy;
		open_cell();
		make_table($newref);
		close_cell($depth);
	} else {	# plain old scalar data
		open_cell();
		$output .= $thingy;
		close_cell(0);
	}
}

sub open_table {
	my $tsref = shift;
	$output .= "\n";
	$output .= "\t" x $depth;
	$output .= $tsref->{BORDER} ? "<table border=0>\n" : "<table border=1>\n";
}

sub close_table {
	$output .= "\t" x $depth;
	$output .= "</table>\n";
}

sub open_row {
	$output .= "\t" x ($depth);
	$output .= "<tr>\n";
	$depth++;
}

sub close_row {
	$depth--;
	$output .= "\t" x ($depth);
	$output .= "</tr>\n";
}

sub open_cell {
	$output .= "\t" x ($depth);
	$output .= "<td>";
	$depth++;
}

sub close_cell {
	my $d = shift;
	$d-- if $d;
	$output .= "\t" x ($d);
	$output .= "</td>\n";
	$depth--;
}
	
sub make_table {
	my $tsref = shift;
	open_table($tsref);
	foreach my $key (sort { 
		if ($tsref->{SORTBY} eq "value") {
			if ($tsref->{ORDER} eq 'asc') {
				${$tsref->{DATA}}{$a} cmp ${$tsref->{DATA}}{$b};
			} else { 
				${$tsref->{DATA}}{$b} cmp ${$tsref->{DATA}}{$a};
			}
		} else {
			if ($tsref->{ORDER} eq 'asc') {
				$a cmp $b;
			} else {
				$b cmp $a;
			}
		}
	} keys %{$tsref->{DATA}}) {
		open_row;
		open_cell;
		$output .= $key;
		close_cell(0);
		recurse_through($tsref, ${$tsref->{DATA}}{$key});
		close_row;
	}
	close_table;
}	

